#!/usr/bin/perl -w
#
# p2tadm.pl - v0.0
# This is a script used to perform maintenance on the message text files
# created by the pkt2txt.pl script.
#  Delete message files by age or by a max number of files per message area.
#  If both selected, then it's done by age first.
#  Also, a renumbering of the message files, starting at one. 
# Copyright (c) 2002 - 2003 Robert James Clay. 
# All Rights Reserved. This is free software;  you can redistribute
# it and/or modify it under the same terms as Perl itself.

# Setup

use strict;
#use File::Copy;  <- will I need this?
use Getopt::Std;

use vars qw/ $opt_p $opt_a $opt_n $opt_d $opt_r $opt_b $opt_l $opt_v $opt_x $opt_h /;

use FTN::Utility qw(&logging);


my ($PACKAREA, $areas,
    $DEBUG, %areas, $progid, $logfile, @files);
 

my $progid = "p2tadm";
my $TEMPDIR = "/tmp/pkt2txt.$$";	# needed?

# No user serviceable parts inside

# &commandline();
getopts('p:d:n:a:b:l:rvxh');

if ($opt_h) {
    &help ();
    exit(1);
}

if ($opt_l) {
    $logfile=$opt_l;  # this needs to be the filename & path
    undef $opt_l;
} else {
    $logfile="pkt2txt.log";	# default log file, as for pkt2txt.pl
}

if ($opt_v) {
    $VERBOSE = $opt_v;	# set debug flag
    undef $opt_v;
    &logging($logfile, $progid, "Verbose flag is set ...");
} else {
    $DEBUG = 0;
}
	
if ($opt_x) {
    $DEBUG = $opt_x;	# set debug flag
    undef $opt_x;
    &logging($logfile, $progid, "Debug flag is set ...");
} else {
    $DEBUG = 0;
}
	
if ($opt_b) {
    $BASEDIR=$opt_b;  # this needs to be the base directory
    undef $opt_b;
} else {
    $BASEDIR="msgs";	# default base dir of text msg directories
}


if ($opt_a) {
    $PACKAREA=$opt_a;  # Areaname (= dir name) or 'all'.
    undef $opt_a;
    &areamaintenance;
} else {
    &allareasmaintenance;	# defaults to All echos
}


# do any neccessary cleanup prior to exiting.

exit;

############################################
# subroutines
############################################

############################################
# allareasmaintenance
############################################
sub allareasmaintenance {
#    $PACKAREA="all";	# defaults to All echos
#  get list of directorys off of base dir &
# then call areamaintenance with $PACKAREA
# set to each one.

    &logging($logfile, $progid, "All areas maintenance, not yet implemented");

    #  Starting at the base message directory, $BASEDIR, read a listing of the
    # msg directories off of that.  Exclude "." & ".." directories.  Call
    # areamaintenance subroutine for each of the msg subdirectories. 

    opendir(BSEDIR, "$BASEDIR") or die "Unable to open $BASEDIR: $!";
	
        my @msgdirs = grep(/[0-9]\.pkt/i, readdir(BSEDIR));

    closedir(BSEDIR);


}

############################################
# areamaintenance
############################################
sub areamaintenance {
    # assumes $PACKAREA variable holds area name 

    # if pack option set, check which type of packing is to be done
    # and call the appropriate subroutine.  Defaults to no packing.
    if ($opt_p) {

        if ($opt_p == "age") {
	    # Pack by opt_d days old.
	    &packbydays();
    
        } elseif ($opt_p == "max") {
	    &packbymaxmsgs();
    	    # Pack to a maximum of opt_n message files per area
	}
    }

    # Renumber message files?
    if ($opt_r) {
	&renumbermsgs();
    }

}
############################################
# pack by maximum number of msgs
############################################
sub packbymaxmsgs {
# Pack message directorys to a maximum of
# opt_n message files per area. defaults to 100

    # assumes $PACKAREA variable holds area name 

    &logging($logfile, $progid, "Packing by a max number not yet implemented");

}

############################################
# Pack by days
############################################
sub packbydays {
# Pack message directories by keeping only those message files
# up to opt_d days old.  defaults to 7 days.

    # assumes $PACKAREA variable holds area name 

    &logging($logfile, $progid, "Packing by age not yet implemented");

}

############################################
# renumbermsgs
############################################
sub renumbermsgs {
# Renumber message files, starting at 1.
# script defaults to no renumbering

    # assumes $PACKAREA variable holds area name 

    &logging($logfile, $progid, "Msg File renumbering not yet implemented");

}

############################################
# findhighmsg
############################################

#- rewrite findhighmsg subroutine to actually find the highest numbered message,
#regardless of how many files are in the message dir or if those message files
#are all numbered sequencially.


sub findhighmsg {
# find the actual numerically highest numbered (base) file name.

    #Set $areas{$area} to high message.  ?
    #or just return the highest numbered filename?
    my $ftype="txt";
	
    #	use glob command instead the grep & readdir?

    opendir(BSEDIR, "$BASEDIR/$area") or die "Unable to open $BASEDIR/$area: $!";
	
        my @txtfiles = grep(/[0-9]\.$ftype/i, readdir(BSEDIR));
	
    closedir(BSEDIR);
	
    if ( $txtfiles[0] ) {	#  if any *.txt files are present
	
	my @bfile = split (/\./, @txtfiles);	#split off just the base names
    
	my @sfiles = sort {$b <=> $a} @bfile;	# reverse numerical sort 
	
#   is it needed to recreate the file name?  or is it just needed
#   to have highest number?  if just need the number, then that
#   is in $sfiles[0].

#	my $last = $sfiles[0];				# highest numbered
	my $last = join (".",$sfiles[0], $ftype);	# recreate file name

        if ($DEBUG) &logging($logfile, $progid, "Highest numbered msg file:  $last");

    }
}

############################################
# Help
############################################
sub help {
    print "\nUsage:  p2tadm.pl [-p max|age] [-n max] [-d days] -b BaseDir [-l logfile] [-x] \n\n";
    print "      -p   'max' or 'age'; defaults to no packing\n";
    print "      -n   Pack messages by max number of messages (defaults to 100) \n";
    print "      -d   Pack messages by numbers of days to keep (defaults to 7)\n";
    print "      -r   Renumber text message files, starting them at 1.\n";
    print "      -a   Name of area to pack, defaults to 'all'\n";
    print "      -b   Base of message text file directories (defaults to 'msgs')\n";
    print "      -l   Log filename (defaults to pkt2txt.log)\n";
    print "\n";
}

