#!/usr/bin/perl -w
#
# pkt2txt.pl - v0.2
# This is a script to toss messages from inbound fidonet packets
# to text files.
# Copyright (c) 2001-2003 Robert James Clay & Russ Johnson.
# All Rights Reserved. This is free software;  you can redistribute
# it and/or modify it under the same terms as Perl itself.

# Setup

use strict;
use File::Copy;
use Getopt::Std;
use vars qw/ $opt_i $opt_o $opt_f $opt_l $opt_e $opt_t $opt_x $opt_h $opt_k /;

use FTN::Utility qw(&logging);

my $progid = "rpkt";

my ($attrib, $area, $datetime, $fromnode, $tonode, $from, $OUTDIR, $outfile, 
    $to, $subj, $msgbody, $msgid, $replyid, $logfile, $FILE,
    $cost, $osep, $orgnet, $destnet, $sec, $min, $hour, $mday, $mon, $year,
    $wday, $orgnode, $destnode, $seenby, $mailer, $origin, $buf, $s, $k, $i,
    $DEBUG, $PKTver, $TYPE, $PKT, $KEEPPKT, $INDIR, $FILENAME, $EXT, 
    $next, $previous, %areas, $MsgTemp, $index, $dir, $echolist,
    $arealist, $description, $thisarea, $newline, $last, 
    $logfilename, $kludges, $org, $dest, $pagespec, @PKTS, @echolist,
    @temp, @lines, @moy, @wdy, @time, @files, @args, @kludges);
 

my $TEMPDIR = "/tmp/pkt2txt.$$";

# Archivers
# my $ZIP = "/usr/bin/zip";
# my $ARC = "/usr/bin/arc";

# Unarchiver
my $UNZIP = "/usr/bin/unzip";
my $UNARC = "/usr/bin/arc";

# How Wide (in characters) to make messages
my $wrap = "80";

# No user serviceable parts inside

&commandline();

&checkformail();

&expandmail();

&tossmail();

&cleanup();

############################################
# subroutines
############################################

############################################
# Check for command line arguments, handle.
############################################
# Add handlers for other command line 
# arguments here
############################################
sub commandline {

    getopts('i:o:f:l:e:txhk');

    if ($opt_h) {
        &help ();
	exit(1);
    }

    if ($opt_x) {
        $DEBUG = $opt_x;	# set debug flag
        undef $opt_x;
        print "\nDebug flag is set ...\n";
    } else {
        $DEBUG = 0;
    }
	
    if ($opt_i) {
        $INDIR=$opt_i;  # this needs to be the filename & path
        undef $opt_i;
    } else {
        $INDIR="inbound";		# default inbound dir
    }

    if ($opt_o) {
        $OUTDIR=$opt_o;  # this needs to be the filename & path
        undef $opt_o;
    } else {
        $OUTDIR="msgs";		# default outbound dir
    }

    if ($opt_e) {
        # Echolist name and location
        @echolist = $opt_e;
#        @echolist = qw($opt_e);  #??
        undef $opt_e;
    } else {
        @echolist ="./backbone.na";	# default filename for echolistfile
    }
    print "\nEcholist info is from: @echolist\n";


    if ($opt_f) {
        $outfile=$opt_f;  # this should be the output file format
        undef $opt_f;
    } else {
        $outfile="text";		# default to text
    }

    if ($opt_l) {
        $logfile=$opt_l;  # this needs to be the filename & path
        undef $opt_l;
    } else {
        $logfile="pkt2txt.log";		# default log file
    }

    if ($opt_k) {
        $KEEPPKT = $opt_k;	# set keep packets flag
        undef $opt_k;
        print "\nKeeping FTN packets...\n";
    } else {
        $KEEPPKT = 0;
    }
	
    if ($opt_t) {
        print "\nContinuing...\n";
        return;
    }
    
}

############################################
# Help
############################################
sub help {
    print "\nUsage:  pkt2txt.pl -i InboundDir -o OutboundDir [-f text|html ] -t /";
    print "                     [-l logfilenamename] [-d] \n\n";
    print "   --toss  -t	Toss Mail\n";
    print "   --debug -x	Debug Mode\n";
    print "           -i   Inbound FTN archive/packet Dir (defaults to 'inbound')\n";
    print "           -f   'text' or 'html' (defaults to 'text')\n";
    print "           -k   keep incoming FTN packets (defaults to deleting them)\n";
    print "           -o   Outbound Dir (defaults to 'msgs')\n";
    print "   --log   -l   Log filename (defaults to pkt2txt.log)\n";
    print "           -e   Echolist file & path\n";
    print "\n";
}

############################################
# Check for new mail
############################################
sub checkformail {
    opendir(DIR, $INDIR);
    @files = grep(/.*\.s[ua][0-9a-z]|mo[0-9a-z]|t[hu][0-9a-z]|we[0-9a-z]|fr[0-9a-z]|pkt/i, readdir(DIR));
    closedir(DIR);
    exit if ($#files == -1);
}

############################################
# If new mail, expand
############################################
sub expandmail {
    # Unarchive mail.
	
    if ($DEBUG) {print "unarchive\n"};	# for testing
	
    foreach $FILE (sort @files) {
    
	# if $FILE = "." or "..", do next file
		
	if ($DEBUG) {print "$INDIR/$FILE\n"};
		
	($FILENAME, $EXT) = split (/\./, "$FILE");

	undef $FILENAME;
				
	if ( "$EXT" ne "pkt" ) {
		open ARCHIVE, "$INDIR/$FILE" or die "Unable to open $FILE: $!\n";
		read ARCHIVE, $TYPE, 2;
				
		if ($TYPE eq "PK") {
			@args = ("$UNZIP", "-q", "$INDIR/$FILE", "-d $TEMPDIR");
		}
		else {
			# Default unarchiver
			@args = ("$UNARC", "x", "$INDIR/$FILE");
		}
			
		!system "@args" or die "@args failed: $?";
			
		close ARCHIVE;
	}
	else {
		if ( ! -d "$TEMPDIR" ) {
			mkdir ("$TEMPDIR", 0750 );
		}
			
		copy("$INDIR/$FILE", "$TEMPDIR/$FILE");
	}
		
	# delete packets if flag is not set
	if (!$KEEPPKT) {
	    unlink ("$INDIR/$FILE");
	}
    }
}

############################################
# Toss expanded packets
############################################
sub tossmail {
    # Toss mail
	
    if ($DEBUG) {print "Tossing...\n"};	# for testing
	
    opendir(TEMPDIR, $TEMPDIR);
#    @PKTS = readdir(TEMPDIR);
    @PKTS = grep(/.*\.pkt/i,readdir(TEMPDIR));  #packet files (testing, 3/28)
    closedir(TEMPDIR);
	
    foreach $PKT (sort @PKTS) {
	if ($DEBUG) {print "Toss messages here\n";
	print "$TEMPDIR/$PKT\n"};
	$PKT = "$TEMPDIR/$PKT";
	open(PKT,$PKT) || &logging($logfile, $progid, "$PKT,$!");
	binmode(PKT);
		
	&logging($logfile, $progid, "Open: $PKT"); 
		
	&newmsgs();
		
	&logging($logfile, $progid, "Deleting: $PKT"); 
		
	unlink($PKT);
    }
}

############################################
# Post New Messages
############################################
sub newmsgs 
{
#    local($to,$from,$subj,$next); #,$msgid);
#    local($buf,$s,$osep,$previous); # $c Removed rlj


    read(PKT,$buf,58);  	#  Ignore packet header

    while (!eof(PKT))		# while note eof on packet
    {
          last if (read(PKT,$buf,14) != 14);
          ($PKTver,$orgnode,$destnode,$orgnet,$destnet,$attrib,$cost) = unpack("SSSSSSS",$buf);

	  undef $PKTver;		#  not used for anything yet - 8/26/01 rjc
	  undef $attrib;		#  not used for anything yet - 8/26/01 rjc

          $osep = $/;                   
          $/ = "\0";                    

          $datetime = <PKT>;         
          if (length($datetime) > 20) 
          {
             $to = substr($datetime,20);
          } else {
             $to = <PKT>;
          }
          $from = <PKT>;
          $subj = <PKT>;

          $to       =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $to       =~ tr/\0-\037/\040-\100/;     # mask control characters
          $from     =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $from     =~ tr/\0-\037/\040-\100/;     # mask control characters
          $datetime =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
          $datetime =~ tr/\0-\037/\040-\100/;     # mask control characters
          $subj     =~ tr/\0-\037/\040-\100/;     # mask control characters

          $s = <PKT>;      
          $/ = $osep;

          $s =~ s/\x8d/\r/g;    
          @lines = split(/\r/,$s);  

          undef $s;     

          next if ($#lines < 0);      

          $area = shift(@lines);          
          $_=$area;
          $area="netmail" if /\//i; 
          $area =~ s/.*://;			# strip "area:"
          $area =~ tr/A-Z/a-z/;			# Lower case area
          
          # is this the first message in this area?
          
          if ( opendir(DIR, "$OUTDIR/$area") ) {
          	closedir(DIR);
          	findhighmsg();
          }
          else {
          	mkdir ("$OUTDIR/$area", 0755 );	
          	findhighmsg();
          }


          for ($i = $k = 0; $i <= $#lines; $i++) 
          {
               if ($lines[$i] =~ /^\001/) 
               {       
                  $kludges[$k++] = splice(@lines,$i,1);
                  redo;
               }
          }
          
    for (;;) {
              $_ = pop(@lines);
              last if ($_ eq "");  
          if (/ \* origin: /i) 
          {
             $origin = substr($_,11);
             last;
          }
          if (/---/) 
          {
             $mailer = $_;
          }
          if (/seen-by/i) 
          {
            $seenby=$_;
          }
    }
      
      
    if ($#lines < 0)           
    {
       @lines = ("[empty message]");
    }
    
    @lines = &wraparray(@lines); 
    
    $areas{$area}++;
    
    $next = $previous = $areas{$area};
        
    $previous--;
    
    $next++;
    
    &buildmsg;
        
    # What output format are we using?
    &printtxt;		# defaults to outputing text
    
    }
}

############################################
# Build message
############################################

sub buildmsg {

    $MsgTemp = "\n";	#  add CR to init of MsgTemp in buildmsg subroutine.
    $kludges = "";

    #&showheader;

    foreach $s (@lines) 
    {
	$s =~ tr/\0-\037/\040-\100/;   
	$s =~ s/\s+$//;                
	$s=~tr/^\*/ /;
	$MsgTemp = $MsgTemp . "$s\n";
    }
    
    foreach $s (@kludges)
    {
	$kludges = $kludges . "$s\n";
    }    
}

############################################
# Print text format messages
############################################

sub printtxt {

    $pagespec = getData( "TEXT" );
	
    &addcontent;
	
    open(MSG,">$OUTDIR/$area/$areas{$area}.txt") ||
    &logging(">$OUTDIR/$area/$areas{$area}.txt",$!);
    binmode(MSG);
      
    print MSG "$pagespec";

    close(MSG);
    undef @lines;    
}

############################################
# Replace placeholders.
############################################

sub addcontent {

    $org = "1:$orgnet/$orgnode\n";
    $dest = "1:$destnet/$destnode\n";

    $pagespec =~ s/<!-- FROM -->/$from/s;
    $pagespec =~ s/<!-- TO -->/$to/s;
    $pagespec =~ s/<!-- SUBJECT -->/$subj/s;
    $pagespec =~ s/<!-- DATETIME -->/$datetime/s;
    $pagespec =~ s/<!-- AREA -->/$area/s;
    $pagespec =~ s/<!-- MESSAGE -->/$MsgTemp/s;	
    $pagespec =~ s/<!-- MAILER -->/$mailer/s if ($mailer);
    $pagespec =~ s/<!-- ORIGIN -->/$origin/s;
    $pagespec =~ s/<!-- SEENBY -->/$seenby/s if ($seenby);
    $pagespec =~ s/<!-- KLUDGES -->/$kludges/s;
    $pagespec =~ s/<!-- NEXT -->/$next/s;
    $pagespec =~ s/<!-- PREVIOUS -->/$previous/s;
}

############################################
# Wrap long lines of text
############################################

sub wraparray {
   @temp="";
   while ($#_ > -1) {
      $_ = shift;
      while (length($_) > $wrap) {
         if ($_ =~ /.{1,$wrap}\s/) {
            push(@temp,$&);
            $_ = $';
         }
         else {
            last;
         }
      }
      push(@temp,$_);
   }
   @temp;
}

############################################
# Put the date in Human format
############################################

sub datetime {
#    local($sec, $min, $hour, $mday, $mon, $year, $wday, @moy, @wdy);
    @moy = ('jan','feb','mar','apr','may','jun',
            'jul','aug','sep','oct','nov','dec');
    @wdy = ('mon','tue','wed','thu','fri','say','sun');
    ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime();
    sprintf("%s, %d %s %d %02d:%02d:%02d",
            $wdy[$wday], $mday, $moy[$mon], $year, $hour, $min, $sec);
}


sub findhighmsg {
# Set $areas{$area} to high message.
#  the way this is currently implemented, this assumes that the total
# count of files is the same as the highest numbered filename.
	
    opendir(OUTDIR, "$OUTDIR/$area") or die "Unable to open $OUTDIR/$area: $!";
	
    my @countfiles = grep(/[0-9]\.*/i, readdir(OUTDIR));
	
    closedir(OUTDIR);
	
    if ( $countfiles[0] ) {
	
	my $last = pop( @countfiles );
	
	my @l = split (/\./, $last);
		
	$areas{$area} = $l[0];
		
#	$areas{$area}-- if ( "$outfile" eq "html" );
    }
}

# --------------------------------------------------------------------
# Function:  getData
# Returns:  section as a String, or undef if no section of the
#   provided name is found.
# Parameters:
#   $section    String with desired section name
# Description:  Allows for named sections of the __DATA__ block of this
#   script to be accessed.
# --------------------------------------------------------------------
{
my %data;
sub getData {
    my ( $section ) = @_;
    if ( ! %data ) {
    	my $block = '';
	while ( 1 ) {
		my $line = <DATA>;
		if ( !defined( $line ) or $line =~ m/^\s*$/ ) {
			if ( $block ne '' ) {
				$block =~ s/\\\n\s*//gs;
				$block =~ s/^([A-Z_]+):(.*)$/$2/s;
				$data{$1} = $block;
				$block = ''
			}
			last if ( !defined( $line ) );
		} else {
			$block .= $line;
		}
	}
	close( DATA );
}
return ( defined( $data{$section} ) ? $data{$section} : undef );
}
}

sub cleanup {

    if ( -d $TEMPDIR ) {
    	rmdir $TEMPDIR or die "Unable to remove $TEMPDIR: $!\n";
    }
}

#####################################################################
# DATA SECTION
#####################################################################
__DATA__

TEXT:
From: <!-- FROM --> 
To: <!-- TO --> 
Subject: <!-- SUBJECT -->
Area: <!-- AREA -->  Date: <!-- DATETIME -->  
=====================================================================
<!-- MESSAGE -->
<!-- MAILER -->
<!-- ORIGIN -->
<!-- SEENBY -->
<!-- KLUDGES -->

